@@ -1,7 +1,11 @@
 import os
 from pathlib import Path
 
-from gi.repository import Gtk, Gdk, Gio, GdkPixbuf
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import Gio
+from gi.repository import GdkPixbuf
+from gi.repository import Pango
 
 from . import APPLICATION_NAME, APPLICATION_ID
 from . import helpers
@@ -24,7 +28,7 @@ class MainWindow(Gtk.ApplicationWindow):
             icon=GdkPixbuf.Pixbuf.new_from_file(
                 data_helpers.find_data_path("images/icon_64.png")
             ),
-            default_width=1000,
+            default_width=700,
             default_height=600,
             resizable=True,
         )
@@ -33,9 +37,7 @@ class MainWindow(Gtk.ApplicationWindow):
 
         self._builder = Gtk.Builder()
         self._builder.set_translation_domain(APPLICATION_ID)
-        self._builder.add_from_file(
-            data_helpers.find_data_path("ui/main-window.glade")
-        )
+        self._builder.add_from_file(data_helpers.find_data_path("ui/main-window.glade"))
         self._builder.connect_signals(self)
 
         header = self._builder.get_object("main_window_header")
@@ -65,9 +67,7 @@ class MainWindow(Gtk.ApplicationWindow):
 
         # Action: win.remove-selected-image
         action = Gio.SimpleAction.new("remove-selected-images", None)
-        action.connect(
-            "activate", self._on_remove_selected_images_action_activated
-        )
+        action.connect("activate", self._on_remove_selected_images_action_activated)
         self.add_action(action)
 
         images_treeview = self._builder.get_object("images_treeview")
@@ -104,7 +104,6 @@ class MainWindow(Gtk.ApplicationWindow):
     def switch_state(self, state):
         app = self.get_application()
 
-        # fmt: off
         if state == app.STATE_MANAGE_IMAGES:
             self._builder.get_object("add_image_button").set_sensitive(True)
             self._builder.get_object("remove_image_button").set_sensitive(True)
@@ -125,7 +124,8 @@ class MainWindow(Gtk.ApplicationWindow):
             self._builder.get_object("jpeg_options").set_sensitive(False)
             self._builder.get_object("webp_options").set_sensitive(False)
             self._builder.get_object("png_options").set_sensitive(False)
-        # fmt: on
+        elif state == app.STATE_SHUTDOWN:
+            self.set_sensitive(False)
 
     def update_interface(self):
         self._updating_interface = True
@@ -167,9 +167,7 @@ class MainWindow(Gtk.ApplicationWindow):
             output_format = None
 
         # [Output image] Update and show output image options
-        output_format_combobox = self._builder.get_object(
-            "output_format_combobox"
-        )
+        output_format_combobox = self._builder.get_object("output_format_combobox")
         if output_format is not None:
             output_format_combobox.set_active(
                 get_supported_output_format_ids().index(output_format)
@@ -187,35 +185,20 @@ class MainWindow(Gtk.ApplicationWindow):
             resize_checkbutton.set_active(False)
             resize_checkbutton.set_inconsistent(True)
 
-        resize_width_adjustment = self._builder.get_object(
-            "resize_width_adjustment"
-        )
+        resize_width_adjustment = self._builder.get_object("resize_width_adjustment")
         resize_width_adjustment.set_value(
-            max(
-                [app.image_store.get(iter_)["resize_width"] for iter_ in iters]
-            )
+            max([app.image_store.get(iter_)["resize_width"] for iter_ in iters])
         )
 
-        resize_width_spinbutton = self._builder.get_object(
-            "resize_width_spinbutton"
-        )
+        resize_width_spinbutton = self._builder.get_object("resize_width_spinbutton")
         resize_width_spinbutton.set_sensitive(resize_checkbutton.get_active())
 
-        resize_height_adjustment = self._builder.get_object(
-            "resize_height_adjustment"
-        )
+        resize_height_adjustment = self._builder.get_object("resize_height_adjustment")
         resize_height_adjustment.set_value(
-            max(
-                [
-                    app.image_store.get(iter_)["resize_height"]
-                    for iter_ in iters
-                ]
-            )
+            max([app.image_store.get(iter_)["resize_height"] for iter_ in iters])
         )
 
-        resize_height_spinbutton = self._builder.get_object(
-            "resize_height_spinbutton"
-        )
+        resize_height_spinbutton = self._builder.get_object("resize_height_spinbutton")
         resize_height_spinbutton.set_sensitive(resize_checkbutton.get_active())
 
         resize_reset_button = self._builder.get_object("resize_reset_button")
@@ -285,8 +268,7 @@ class MainWindow(Gtk.ApplicationWindow):
 
         iters = self.get_selected_image_iters()
         for iter_ in iters:
-            app = self.get_application()
-            app.image_store.remove(iter_)
+            app.remove_image(iter_)
 
     def get_selected_image_iters(self):
         treeview_images = self._builder.get_object("images_treeview")
@@ -299,56 +281,101 @@ class MainWindow(Gtk.ApplicationWindow):
     def _prepare_treeview(self):
         app = self.get_application()
 
-        DISPLAYED_FIELDS = [
-            "status_display",
-            "preview",
-            "input_file_display",
-            "input_size_display",
-            "separator",
-            "output_file_display",
-            "output_format_display",
-            "output_size_display",
-        ]
-
         treeview_images = self._builder.get_object("images_treeview")
         treeview_images.set_model(app.image_store.gtk_list_store)
 
-        for field_name in DISPLAYED_FIELDS:
-            field_type = app.image_store.FIELDS[field_name]["type"]
+        # Preview
+        pixbuf = Gtk.CellRendererPixbuf()
+        column = Gtk.TreeViewColumn("", pixbuf)
+        column.add_attribute(
+            pixbuf,
+            "pixbuf",
+            app.image_store.FIELDS["preview"]["id"],
+        )
+        treeview_images.append_column(column)
 
-            if field_type is str:
-                treeview_images.append_column(
-                    Gtk.TreeViewColumn(
-                        app.image_store.FIELDS[field_name]["label"],
-                        Gtk.CellRendererText(),
-                        text=app.image_store.FIELDS[field_name]["id"],
-                    ),
-                )
-            elif field_type is GdkPixbuf.Pixbuf:
-                pixbuf = Gtk.CellRendererPixbuf()
-                column = Gtk.TreeViewColumn(
-                    app.image_store.FIELDS[field_name]["label"],
-                    pixbuf,
-                )
-                column.add_attribute(
-                    pixbuf,
-                    "pixbuf",
-                    app.image_store.FIELDS[field_name]["id"],
-                )
-                treeview_images.append_column(column)
-            else:
-                raise TypeError(
-                    "Unsupported field type '%s'" % str(field_type)
-                )
+        # Input file / Output file
+        column = Gtk.TreeViewColumn(_("Input Image / Output Image"))
+        column.set_expand(True)
+        column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+        column.set_resizable(True)
+        column.set_min_width(100)
+        column.get_area().set_orientation(Gtk.Orientation.VERTICAL)
+        treeview_images.append_column(column)
+
+        renderer = Gtk.CellRendererText(
+            weight=700,
+            yalign=1.0,
+            ypad=0,
+            ellipsize=Pango.EllipsizeMode.MIDDLE,
+        )
+        column.pack_start(renderer, True)
+        column.add_attribute(
+            renderer,
+            "text",
+            app.image_store.FIELDS["input_file_display"]["id"],
+        )
+
+        renderer = Gtk.CellRendererText(
+            weight=400,
+            scale=0.75,
+            yalign=0.0,
+            ypad=0,
+            ellipsize=Pango.EllipsizeMode.MIDDLE,
+        )
+        column.pack_start(renderer, True)
+        column.add_attribute(
+            renderer,
+            "text",
+            app.image_store.FIELDS["output_file_display"]["id"],
+        )
+
+        # Output Format
+        treeview_images.append_column(
+            Gtk.TreeViewColumn(
+                _("Output Format"),
+                Gtk.CellRendererText(),
+                markup=app.image_store.FIELDS["output_format_display"]["id"],
+            ),
+        )
+
+        # Input Size
+        treeview_images.append_column(
+            Gtk.TreeViewColumn(
+                _("Input Size"),
+                Gtk.CellRendererText(),
+                text=app.image_store.FIELDS["input_size_display"]["id"],
+            ),
+        )
+
+        # Output Size / Status
+        column = Gtk.TreeViewColumn(_("Output Size"))
+        treeview_images.append_column(column)
+
+        renderer = Gtk.CellRendererText()
+        column.pack_start(renderer, True)
+        column.add_attribute(
+            renderer,
+            "markup",
+            app.image_store.FIELDS["output_size_display"]["id"],
+        )
+
+        renderer = Gtk.CellRendererText(
+            scale=1.2,
+        )
+        column.pack_start(renderer, True)
+        column.add_attribute(
+            renderer,
+            "text",
+            app.image_store.FIELDS["optimization_success"]["id"],
+        )
 
         # Enable multi-selection
         selection = treeview_images.get_selection()
         selection.set_mode(Gtk.SelectionMode.MULTIPLE)
 
     def _prepare_format_combobox(self):
-        output_format_combobox = self._builder.get_object(
-            "output_format_combobox"
-        )
+        output_format_combobox = self._builder.get_object("output_format_combobox")
 
         for output_format in get_supported_output_format_names():
             output_format_combobox.append_text(output_format)
@@ -357,9 +384,7 @@ class MainWindow(Gtk.ApplicationWindow):
         app = self.get_application()
         app.quit()
 
-    def _on_drag_data_received(
-        self, widget, drag_context, x, y, data, info, time
-    ):
+    def _on_drag_data_received(self, widget, drag_context, x, y, data, info, time):
         app = self.get_application()
 
         def _add_path(paths):
@@ -382,9 +407,7 @@ class MainWindow(Gtk.ApplicationWindow):
             return
 
         app = self.get_application()
-        output_format_combobox = self._builder.get_object(
-            "output_format_combobox"
-        )
+        output_format_combobox = self._builder.get_object("output_format_combobox")
 
         # No format selected
         if output_format_combobox.get_active() == -1:
@@ -409,9 +432,7 @@ class MainWindow(Gtk.ApplicationWindow):
         iters = self.get_selected_image_iters()
 
         for iter_ in iters:
-            app.image_store.update(
-                iter_, resize_enabled=checkbutton.get_active()
-            )
+            app.image_store.update(iter_, resize_enabled=checkbutton.get_active())
             app.image_store.reset_status(iter_)
 
         self.update_interface()
@@ -424,9 +445,7 @@ class MainWindow(Gtk.ApplicationWindow):
         iters = self.get_selected_image_iters()
 
         for iter_ in iters:
-            app.image_store.update(
-                iter_, resize_width=round(adjustment.get_value())
-            )
+            app.image_store.update(iter_, resize_width=round(adjustment.get_value()))
             app.image_store.reset_status(iter_)
 
         self.update_interface()
@@ -439,9 +458,7 @@ class MainWindow(Gtk.ApplicationWindow):
         iters = self.get_selected_image_iters()
 
         for iter_ in iters:
-            app.image_store.update(
-                iter_, resize_height=round(adjustment.get_value())
-            )
+            app.image_store.update(iter_, resize_height=round(adjustment.get_value()))
             app.image_store.reset_status(iter_)
 
         self.update_interface()
@@ -495,9 +512,7 @@ class MainWindow(Gtk.ApplicationWindow):
 
         if len(iters) == 1:
             output_file_entry = self._builder.get_object("output_file_entry")
-            output_file_entry.set_text(
-                app.image_store.get(iters[0])["output_file"]
-            )
+            output_file_entry.set_text(app.image_store.get(iters[0])["output_file"])
 
     def _on_output_pattern_subfolder_modelbutton_clicked(self, widget):
         app = self.get_application()
@@ -513,9 +528,7 @@ class MainWindow(Gtk.ApplicationWindow):
 
         if len(iters) == 1:
             output_file_entry = self._builder.get_object("output_file_entry")
-            output_file_entry.set_text(
-                app.image_store.get(iters[0])["output_file"]
-            )
+            output_file_entry.set_text(app.image_store.get(iters[0])["output_file"])
 
     def _on_output_pattern_custom_modelbutton_clicked(self, widget):
         app = self.get_application()
@@ -541,9 +554,7 @@ class MainWindow(Gtk.ApplicationWindow):
 
         if len(iters) == 1:
             output_file_entry = self._builder.get_object("output_file_entry")
-            output_file_entry.set_text(
-                app.image_store.get(iters[0])["output_file"]
-            )
+            output_file_entry.set_text(app.image_store.get(iters[0])["output_file"])
 
     def _on_output_path_browse_modelbutton_clicked(self, widget):
         app = self.get_application()
@@ -561,11 +572,9 @@ class MainWindow(Gtk.ApplicationWindow):
                 use_output_pattern=False,
             )
             output_file_entry = self._builder.get_object("output_file_entry")
-            output_file_entry.set_text(
-                app.image_store.get(iter_)["output_file"]
-            )
+            output_file_entry.set_text(app.image_store.get(iter_)["output_file"])
 
-    def _on_jpeg_quality_adjustement_value_changed(self, adjustment):
+    def _on_jpeg_quality_adjustment_value_changed(self, adjustment):
         if self._updating_interface:
             return
 
@@ -573,12 +582,10 @@ class MainWindow(Gtk.ApplicationWindow):
         iters = self.get_selected_image_iters()
 
         for iter_ in iters:
-            app.image_store.update(
-                iter_, jpeg_quality=round(adjustment.get_value())
-            )
+            app.image_store.update(iter_, jpeg_quality=round(adjustment.get_value()))
             app.image_store.reset_status(iter_)
 
-    def _on_webp_quality_adjustement_value_changed(self, adjustment):
+    def _on_webp_quality_adjustment_value_changed(self, adjustment):
         if self._updating_interface:
             return
 
@@ -586,9 +593,7 @@ class MainWindow(Gtk.ApplicationWindow):
         iters = self.get_selected_image_iters()
 
         for iter_ in iters:
-            app.image_store.update(
-                iter_, webp_quality=round(adjustment.get_value())
-            )
+            app.image_store.update(iter_, webp_quality=round(adjustment.get_value()))
             app.image_store.reset_status(iter_)
 
     def _on_png_slow_optimization_checkbutton_toggled(self, checkbutton):
@@ -608,3 +613,11 @@ class MainWindow(Gtk.ApplicationWindow):
 
     def _on_remove_selected_images_action_activated(self, action, param):
         self.remove_selected_images()
+
+    def _on_images_treeview_button_pressed(self, treeview, event):
+        if event.type != Gdk.EventType._2BUTTON_PRESS:
+            return
+        app = self.get_application()
+        iters = self.get_selected_image_iters()
+        for iter_ in iters:
+            helpers.open_file(app.image_store.get(iter_)["input_file"])
